@@ -17,6 +17,8 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import json
+import logging
+import typing
 import unittest.mock
 
 import pytest
@@ -100,6 +102,7 @@ def test__main(
         mqtt_password=expected_password,
         retry_count=expected_retry_count,
         device_passwords={},
+        fetch_device_info=False,
     )
 
 
@@ -144,6 +147,7 @@ def test__main_mqtt_password_file(
         mqtt_password=expected_password,
         retry_count=3,
         device_passwords={},
+        fetch_device_info=False,
     )
 
 
@@ -202,4 +206,62 @@ def test__main_device_password_file(tmpdir, device_passwords):
         mqtt_password=None,
         retry_count=3,
         device_passwords=device_passwords,
+        fetch_device_info=False,
+    )
+
+
+def test__main_fetch_device_info():
+    with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
+        "sys.argv",
+        [
+            "",
+            "--mqtt-host",
+            "localhost",
+        ],
+    ):
+        # pylint: disable=protected-access
+        switchbot_mqtt._main()
+    default_kwargs = dict(
+        mqtt_host="localhost",
+        mqtt_port=1883,
+        mqtt_username=None,
+        mqtt_password=None,
+        retry_count=3,
+        device_passwords={},
+    )
+    run_mock.assert_called_once_with(fetch_device_info=False, **default_kwargs)
+    with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
+        "sys.argv",
+        ["", "--mqtt-host", "localhost", "--fetch-device-info"],
+    ):
+        # pylint: disable=protected-access
+        switchbot_mqtt._main()
+    run_mock.assert_called_once_with(fetch_device_info=True, **default_kwargs)
+
+
+@pytest.mark.parametrize(
+    ("additional_argv", "root_log_level", "log_format"),
+    [
+        ([], logging.INFO, "%(message)s"),
+        (
+            ["--debug"],
+            logging.DEBUG,
+            "%(asctime)s:%(levelname)s:%(name)s:%(funcName)s:%(message)s",
+        ),
+    ],
+)
+def test__main_log_config(
+    additional_argv: typing.List[str], root_log_level: int, log_format: str
+):
+    with unittest.mock.patch(
+        "sys.argv", ["", "--mqtt-host", "localhost"] + additional_argv
+    ), unittest.mock.patch(
+        "logging.basicConfig"
+    ) as logging_basic_config_mock, unittest.mock.patch(
+        "switchbot_mqtt._run"
+    ):
+        # pylint: disable=protected-access
+        switchbot_mqtt._main()
+    logging_basic_config_mock.assert_called_once_with(
+        level=root_log_level, format=log_format, datefmt="%Y-%m-%dT%H:%M:%S%z"
     )
